
/*************************************************
****          VEGA - Command parser           ****
**** Copyright 1996-2002, Alessandro Pedretti ****
*************************************************/


#ifndef __COMPAR_H
#define __COMPAR_H

#ifndef __COMERR_H
#  include "comerr.h"
#endif

/**** Get variables ****/

enum {
  VG_GETVAR_CURLANG,
  VG_GETVAR_DATADIR,
  VG_GETVAR_ERRCODE,
  VG_GETVAR_ERRSTR,
  VG_GETVAR_TOTATM,
  VG_GETVAR_VEGADIR,
  VG_GETVAR_VERSION,

  VG_GETVAR_PRFLANG,

  VG_GETVAR_TRJBESTENERGY,
  VG_GETVAR_TRJBESTFRM,
  VG_GETVAR_TRJCURFRM,
  VG_GETVAR_TRJSIMTIME,
  VG_GETVAR_TRJSTARTTIME,
  VG_GETVAR_TRJTEMP,
  VG_GETVAR_TRJTIMESTEP,
  VG_GETVAR_TRJTOTFRM,
  VG_GETVAR_TRJWORSEENERGY,
  VG_GETVAR_TRJWORSEFRM,

#ifdef __VG_OPENGL
  VG_GETVAR_PORTNUM,
  VG_GETVAR_ROTX    ,  VG_GETVAR_ROTY   , VG_GETVAR_ROTZ,
  VG_GETVAR_SCALE,
  VG_GETVAR_TRANSX  , VG_GETVAR_TRANSY  , VG_GETVAR_TRANSZ,
  VG_GETVAR_WINSIZEX, VG_GETVAR_WINSIZEY,
  VG_GETVAR_WINPOSX , VG_GETVAR_WINPOSY
#endif
};

/**** Standard command codes ****/

enum {
  VG_PARCOM_NOTHING   =  0,      /* Nothing                               */
  VG_PARCOM_CHARGE,              /* Assign the atom charges               */
  VG_PARCOM_GET,                 /* Get a value                           */
  VG_PARCOM_MSGERRMODE,          /* Message error mode                    */
  VG_PARCOM_FORCEFIELD,          /* Assign the atom types                 */
  VG_PARCOM_NEW,                 /* New                                   */
  VG_PARCOM_OPENMOL,             /* Open a molecule                       */
  VG_PARCOM_SAVEMOL,             /* Save a molecule                       */
  VG_PARCOM_TRJANIMPLAY,         /* Trajectory animation start            */
  VG_PARCOM_TRJANIMSET,          /* Trajectory animation settings         */
  VG_PARCOM_TRJANIMSPEED,        /* Trajectory animation speed            */
  VG_PARCOM_TRJANIMSTOP,         /* Trajectory animation stop             */
  VG_PARCOM_TRJOPEN,             /* Open a trajectory file                */
  VG_PARCOM_TRJSEL,              /* Select a specified frame              */
  VG_PARCOM_TRJSELFIRST,         /* Select the first frame                */
  VG_PARCOM_TRJSELLAST,          /* Select the last frame                 */
};


/**** OpenGL command codes ****/

#ifdef __VG_OPENGL
enum {
  VG_PARCOM_ANTIALIAS = 101,     /* Anti aliasing                         */
  VG_PARCOM_CLIPCOPY,            /* Copy data into the clipboard          */
  VG_PARCOM_COLOR,               /* Set the color drawing                 */
  VG_PARCOM_CPKPROP,             /* CPK properties                        */
  VG_PARCOM_DEPTHCUE,            /* Depth cue                             */
  VG_PARCOM_LIQPROP,             /* Liquorice properties                  */
  VG_PARCOM_PLUGCALL,            /* Plugin: start the plugin              */
  VG_PARCOM_PLUGABOUT,           /* Plugin: open the about dialog         */
  VG_PARCOM_PLUGCONFIG,          /* Plugin: open the configuration dialog */
  VG_PARCOM_PLUGINGET,           /* Equivalent to Get, but faster         */
  VG_PARCOM_PLUGHELP,            /* Plugin: open the help dialog          */
  VG_PARCOM_REFRESH,             /* Refresh the OGL window                */
  VG_PARCOM_RESET,               /* Reset                                 */
  VG_PARCOM_ROTATE,              /* Rotate                                */
  VG_PARCOM_SELECT,              /* Select atom(s)                        */
  VG_PARCOM_SELRANGE,            /* Select range                          */
  VG_PARCOM_SONGPLAY,            /* Play a song                           */
  VG_PARCOM_SONGSTOP,            /* Stop song                             */
  VG_PARCOM_SONGVOL,             /* Song volume                           */
  VG_PARCOM_TRANSLATE,           /* Translate                             */
  VG_PARCOM_UNSELECT,            /* Unselect atom(s)                      */
  VG_PARCOM_UNSELRANGE,          /* Unselect range                        */
  VG_PARCOM_VDWPROP,             /* Van der Waals properties              */
  VG_PARCOM_WIREPROP,            /* Wireframe properties                  */
  VG_PARCOM_ZCLIP,               /* Z-Clip value                          */
  VG_PARCOM_ZOOM                 /* Zoom                                  */
};
#endif

/**** Parameter types ****/

#define  VG_PARTYPE_UNK         0
#define  VG_PARTYPE_BOOL        1
#define  VG_PARTYPE_CHAR        2
#define  VG_PARTYPE_FLOAT       3
#define  VG_PARTYPE_INT         4
#define  VG_PARTYPE_UNSIGNED    8
#define  VG_PARTYPE_NOZERO     16
#define  VG_PARTYPE_MAX        32
#define  VG_PARTYPE_MIN        64
#define  VG_PARTYPE_CMTX      128

#define  VG_PARTYPE_MCHAR     (VG_PARTYPE_CHAR|VG_PARTYPE_CMTX)
#define  VG_PARTYPE_UFLOAT    (VG_PARTYPE_FLOAT|VG_PARTYPE_UNSIGNED)
#define  VG_PARTYPE_RUFLOAT   (VG_PARTYPE_FLOAT|VG_PARTYPE_UNSIGNED|VG_PARTYPE_MAX|VG_PARTYPE_MIN)
#define  VG_PARTYPE_UINT      (VG_PARTYPE_INT|VG_PARTYPE_UNSIGNED)
#define  VG_PARTYPE_RUINT     (VG_PARTYPE_INT|VG_PARTYPE_UNSIGNED|VG_PARTYPE_MAX|VG_PARTYPE_MIN)
#define  VG_PARTYPE_ZUINT     (VG_PARTYPE_INT|VG_PARTYPE_UNSIGNED|VG_PARTYPE_NOZERO)

/**** Command flags ****/

#define  VG_COMFLG_NOTHING      0       /* No special flags            */
#define  VG_COMFLG_OPENGL       1       /* OpenGL command              */
#define  VG_COMFLG_MOL          2       /* Molecule required           */
#define  VG_COMFLG_TRJ          4       /* Trajectory required         */

#define  VG_COMFLG_MOGL         (VG_COMFLG_OPENGL|VG_COMFLG_MOL)

/**** Structures ****/

typedef union {
  VG_LONG               L;
  VG_ULONG              UL;
  float                 F;
  char                  *C;
  VG_BOOL               B;
} VG_MULTI;

/**** Parameter (see command structure) ****/

typedef struct __VG_PARAM {
  struct __VG_PARAM     *Ptr;
  VG_UBYTE              Type;
  VG_MULTI              Data;
} VG_PARAM;

/**** Command structure ****/

typedef struct __VG_COMM {
  struct __VG_COMM      *Ptr;
  VG_UWORD              Id;
  VG_UWORD              NumPar;
  VG_PARAM              *ParList;
} VG_COMM;

/**** Argument infos ****/

typedef struct {
  VG_UBYTE              ArgType;        /* Argument type       */
  VG_MULTI              Min;            /* Min value           */
  VG_MULTI              Max;            /* Max value           */
} VG_ARG_INFO;

/**** Commans infos ****/

typedef struct {
  VG_UWORD              Id;             /* Command ID          */
  VG_UWORD              Flags;          /* Flags               */
  VG_UWORD              NumArgs;        /* Number of arguments */
  char                  *Name;          /* Command Name        */
  VG_ARG_INFO           *ArgTypes;      /* Argument types      */
} VG_COMM_INFO;


/**** Command table ****/

#ifdef __COMPAR

/**** Generic templates ****/

static const VG_ARG_INFO VgInf_Bool1          =  {VG_PARTYPE_BOOL   , 0, 0};
static const VG_ARG_INFO VgInf_Char1          =  {VG_PARTYPE_CHAR   , 0, 0};
static const VG_ARG_INFO VgInf_Float3[]       = {{VG_PARTYPE_FLOAT  , 0, 0},
                                                 {VG_PARTYPE_FLOAT  , 0, 0},
                                                 {VG_PARTYPE_FLOAT  , 0, 0}};
static const VG_ARG_INFO VgInf_Zuint1         =  {VG_PARTYPE_ZUINT  , 0, 0};

/**** Specific templates ****/

static const VG_ARG_INFO VgInf_FField[]       = {{VG_PARTYPE_CHAR   , 0, 0},
                                                 {VG_PARTYPE_BOOL   , 0, 0}};
static const VG_ARG_INFO VgInf_Get            = {VG_PARTYPE_MCHAR   , 0, (VG_ULONG) \
                                                 "CURLANG\0DATADIR\0ERRCODE\0ERRSTR\0TOTATM\0VEGADIR\0VERSION\0"
                                                 "PRFLANG\0"
                                                 "TRJBESTENERGY\0TRJBESTFRM\0TRJCURFRM\0TRJSIMTIME\0TRJSTARTTIME\0"
                                                 "TRJTEMP\0TRJTIMESTEP\0TRJTOTFRM\0TRJWORSEENERGY\0TRJWORSEFRM\0"
#ifdef __VG_OPENGL
                                                 "PORTNUM\0ROTX\0ROTY\0ROTZ\0SCALE\0TRANSX\0TRANSY\0TRANSZ\0"
                                                 "WINSIZEX\0WINSIZEY\0WINPOSX\0WINPOSY\0"
#endif
                                                };
static const VG_ARG_INFO VgInf_MsgErrMode     =  {VG_PARTYPE_MCHAR  , 0, (VG_ULONG)"CONSOLE\0WINDOW\0QUIET\0"};
static const VG_ARG_INFO VgInf_SaveMol[]      = {{VG_PARTYPE_CHAR   , 0, 0},
                                                 {VG_PARTYPE_CHAR   , 0, 0},
                                                 {VG_PARTYPE_CHAR   , 0, 0},
                                                 {VG_PARTYPE_BOOL   , 0, 0}};
static const VG_ARG_INFO VgInf_TrjAnimSet[]   = {{VG_PARTYPE_UINT   , 0, 0},
                                                 {VG_PARTYPE_UINT   , 0, 0},
                                                 {VG_PARTYPE_BOOL   , 0, 0}};
static const VG_ARG_INFO VgInf_TrjAnimSpeed[] = {{VG_PARTYPE_RUINT  , (VG_ULONG)0 , (VG_ULONG)500},
                                                 {VG_PARTYPE_UINT   , 0, 0}};
static const VG_ARG_INFO VgInf_TrjOpen[]      = {{VG_PARTYPE_CHAR   , 0, 0},
                                                 {VG_PARTYPE_BOOL   , 0, 0}};

/**** OpenGL Specific templates ****/

#ifdef __VG_OPENGL
static const VG_ARG_INFO VgInf_Copy           = {VG_PARTYPE_MCHAR   , 0, (VG_ULONG)"VEGA\0BITMAP\0" \
                                                 "BIOSYM\0MOL2\0MOPAC\0PDB\0PDBF\0XYZ\0"};
static const VG_ARG_INFO VgInf_CpkProp[]      = {{VG_PARTYPE_RUINT  , (VG_ULONG)8 , (VG_ULONG)24},
                                                 {VG_PARTYPE_RUINT  , (VG_ULONG)10, (VG_ULONG)50},
                                                 {VG_PARTYPE_RUINT  , (VG_ULONG)5 , (VG_ULONG)20},
                                                 {VG_PARTYPE_RUFLOAT, 0x3d4ccccd  , 0x3f000000  }};
static const VG_ARG_INFO VgInf_LiqProp[]      = {{VG_PARTYPE_RUINT  , (VG_ULONG)8 , (VG_ULONG)24},
                                                 {VG_PARTYPE_RUINT  , (VG_ULONG)5 , (VG_ULONG)20},
                                                 {VG_PARTYPE_RUFLOAT, 0x3d4ccccd  , 0x3f000000  }};
static const VG_ARG_INFO VgInf_PlgCall[]      = {{VG_PARTYPE_CHAR   , 0, 0},
                                                 {VG_PARTYPE_RUINT  , (VG_ULONG)1, (VG_ULONG)100}};
static const VG_ARG_INFO VgInf_SelRange[]     = {{VG_PARTYPE_MCHAR  , 0, (VG_ULONG)"ATOM\0RESIDUE\0"},
                                                 {VG_PARTYPE_ZUINT  , 0, 0},
                                                 {VG_PARTYPE_ZUINT  , 0, 0}};

static const VG_ARG_INFO VgInf_SongVol        =  {VG_PARTYPE_RUINT  , (VG_ULONG)0, (VG_ULONG)255};
static const VG_ARG_INFO VgInf_VdwProp[]      = {{VG_PARTYPE_RUINT  , (VG_ULONG)8, (VG_ULONG)24},
                                                 {VG_PARTYPE_RUINT  , (VG_ULONG)1, (VG_ULONG)4}};
static const VG_ARG_INFO VgInf_WireProp[]     = {{VG_PARTYPE_RUINT  , (VG_ULONG)1, (VG_ULONG)5},
                                                 {VG_PARTYPE_BOOL   , 0, 0}};
static const VG_ARG_INFO VgInf_Zclip          =  {VG_PARTYPE_RUINT  , (VG_ULONG)1, (VG_ULONG)300};
static const VG_ARG_INFO VgInf_Zoom           =  {VG_PARTYPE_UFLOAT , 0, 0};
#endif

/**** Command matrix ****/

static const VG_COMM_INFO    ComInfo[] = {
  {VG_PARCOM_CHARGE        , VG_COMFLG_MOL    , 1, "CHARGE"       , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_FORCEFIELD    , VG_COMFLG_MOL    , 2, "FORCEFIELD"   , (VG_ARG_INFO *)&VgInf_FField       },
  {VG_PARCOM_GET           , VG_COMFLG_NOTHING, 1, "GET"          , (VG_ARG_INFO *)&VgInf_Get          },
  {VG_PARCOM_MSGERRMODE    , VG_COMFLG_NOTHING, 1, "MSGERRMODE"   , (VG_ARG_INFO *)&VgInf_MsgErrMode   },
  {VG_PARCOM_OPENMOL       , VG_COMFLG_NOTHING, 1, "OPEN"         , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_NEW           , VG_COMFLG_NOTHING, 0, "NEW"          , (VG_ARG_INFO *)NULL                },
  {VG_PARCOM_SAVEMOL       , VG_COMFLG_MOL    , 4, "SAVE"         , (VG_ARG_INFO *)&VgInf_SaveMol      },
  {VG_PARCOM_TRJOPEN       , VG_COMFLG_NOTHING, 2, "TRJOPEN"      , (VG_ARG_INFO *)&VgInf_TrjOpen      },
  {VG_PARCOM_TRJSEL        , VG_COMFLG_TRJ    , 1, "TRJSEL"       , (VG_ARG_INFO *)&VgInf_Zuint1       },
  {VG_PARCOM_TRJSELFIRST   , VG_COMFLG_TRJ    , 0, "TRJSELFIRST"  , (VG_ARG_INFO *)NULL                },
  {VG_PARCOM_TRJSELLAST    , VG_COMFLG_TRJ    , 0, "TRJSELLAST"   , (VG_ARG_INFO *)NULL                },

#ifdef __VG_OPENGL
  {VG_PARCOM_ANTIALIAS     , VG_COMFLG_OPENGL , 1, "ANTIALIAS"    , (VG_ARG_INFO *)&VgInf_Bool1        },
  {VG_PARCOM_CLIPCOPY      , VG_COMFLG_MOGL   , 1, "COPY"         , (VG_ARG_INFO *)&VgInf_Copy         },
  {VG_PARCOM_COLOR         , VG_COMFLG_MOGL   , 1, "COLOR"        , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_CPKPROP       , VG_COMFLG_OPENGL , 4, "CPKPROP"      , (VG_ARG_INFO *)&VgInf_CpkProp      },
  {VG_PARCOM_DEPTHCUE      , VG_COMFLG_OPENGL , 1, "DEPTHCUE"     , (VG_ARG_INFO *)&VgInf_Bool1        },
  {VG_PARCOM_LIQPROP       , VG_COMFLG_OPENGL , 3, "LIQPROP"      , (VG_ARG_INFO *)&VgInf_LiqProp      },
  {VG_PARCOM_PLUGABOUT     , VG_COMFLG_OPENGL , 1, "PLUGINABOUT"  , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_PLUGCALL      , VG_COMFLG_OPENGL , 2, "PLUGINCALL"   , (VG_ARG_INFO *)&VgInf_PlgCall      },
  {VG_PARCOM_PLUGCONFIG    , VG_COMFLG_OPENGL , 1, "PLUGINCONFIG" , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_PLUGINGET     , VG_COMFLG_NOTHING, 1, "PLUGINGET"    , (VG_ARG_INFO *)&VgInf_Get          },
  {VG_PARCOM_PLUGHELP      , VG_COMFLG_OPENGL , 1, "PLUGINHELP"   , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_REFRESH       , VG_COMFLG_OPENGL , 0, "REFRESH"      , (VG_ARG_INFO *)NULL                },
  {VG_PARCOM_RESET         , VG_COMFLG_OPENGL , 1, "RESET"        , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_ROTATE        , VG_COMFLG_MOGL   , 3, "ROTATE"       , (VG_ARG_INFO *)&VgInf_Float3       },
  {VG_PARCOM_SELECT        , VG_COMFLG_MOGL   , 1, "SELECT"       , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_SELRANGE      , VG_COMFLG_MOGL   , 3, "SELRANGE"     , (VG_ARG_INFO *)&VgInf_SelRange     },
  {VG_PARCOM_SONGPLAY      , VG_COMFLG_OPENGL , 1, "SONGPLAY"     , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_SONGSTOP      , VG_COMFLG_OPENGL , 0, "SONGSTOP"     , (VG_ARG_INFO *)NULL                },
  {VG_PARCOM_SONGVOL       , VG_COMFLG_OPENGL , 1, "SONGVOL"      , (VG_ARG_INFO *)&VgInf_SongVol      },
  {VG_PARCOM_TRANSLATE     , VG_COMFLG_MOGL   , 3, "TRANSLATE"    , (VG_ARG_INFO *)&VgInf_Float3       },
  {VG_PARCOM_TRJANIMPLAY   , VG_COMFLG_TRJ    , 0, "TRJANIMPLAY"  , (VG_ARG_INFO *)NULL                },
  {VG_PARCOM_TRJANIMSET    , VG_COMFLG_TRJ    , 3, "TRJANIMSET"   , (VG_ARG_INFO *)&VgInf_TrjAnimSet   },
  {VG_PARCOM_TRJANIMSPEED  , VG_COMFLG_TRJ    , 2, "TRJANIMSPEED" , (VG_ARG_INFO *)&VgInf_TrjAnimSpeed },
  {VG_PARCOM_TRJANIMSTOP   , VG_COMFLG_TRJ    , 0, "TRJANIMSTOP"  , (VG_ARG_INFO *)NULL                },
  {VG_PARCOM_UNSELECT      , VG_COMFLG_MOGL   , 1, "UNSELECT"     , (VG_ARG_INFO *)&VgInf_Char1        },
  {VG_PARCOM_UNSELRANGE    , VG_COMFLG_MOGL   , 3, "UNSELRANGE"   , (VG_ARG_INFO *)&VgInf_SelRange     },
  {VG_PARCOM_VDWPROP       , VG_COMFLG_OPENGL , 2, "VDWPROP"      , (VG_ARG_INFO *)&VgInf_VdwProp      },
  {VG_PARCOM_WIREPROP      , VG_COMFLG_OPENGL , 2, "WIREPROP"     , (VG_ARG_INFO *)&VgInf_WireProp     },
  {VG_PARCOM_ZCLIP         , VG_COMFLG_OPENGL , 1, "ZCLIP"        , (VG_ARG_INFO *)&VgInf_Zclip        },
  {VG_PARCOM_ZOOM          , VG_COMFLG_MOGL   , 1, "ZOOM"         , (VG_ARG_INFO *)&VgInf_Zoom         },
#endif

  {VG_PARCOM_NOTHING   , VG_COMFLG_NOTHING, 0, NULL           , (VG_ARG_INFO *)NULL              }
};
#endif


/**** Prototypes ****/

#  ifdef __cplusplus
extern "C" {
#  endif

VG_BOOL  ChkArgType(char *, VG_ULONG, VG_UBYTE);
void     ComExe(VG_COMM *);
VG_ULONG CountArgStr(char *);
char     *GetArgStr(char *, VG_ULONG *, char **);
void     FreeCom(VG_COMM *);
VG_COMM  *ParseStr(char *);

#  ifdef __cplusplus
}
#  endif
#endif

