
/*************************************************
****   VEGA - High level trajectory analyzer  ****
**** Copyright 1996-2002, Alessandro Pedretti ****
*************************************************/


#ifdef __WIN32__
#include <windows.h>
#include <GL\gl.h>
#include <conio.h>
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"
#include "formats.h"

#ifdef WIN32
#include "gl_global.h"
#endif


/**** Calculate a property ****/

float *TrjCalc(TRJINFO *Info, TRJPAR *Par, ATOMO *InizAtm, VG_ULONG MolAtm)
{
  ATOMO                 *Frm[MAXTRJATM];
  char                  *ProcFrmTxt;
  float                 *Iniz, *Ene;
  float                 EneMin, EneMax, MaxRad;
  VG_ULONG              Nu, YPos;
  XYZ                   BoxMin, BoxMax;
  register ATOMO        *Atm;
  register VG_ULONG     k;

  float                 *VdwRad  = NULL;
  double                Media    = 0.0;
  double                Media2   = 0.0;
  VG_BOOL               Ret      = TRUE;
  VG_ULONG              FrmMin   = 1;
  VG_ULONG              FrmMax   = 1;
  XYZ                   *SfUn    = NULL;

#ifdef VLOGP
  float                 *Contrib = NULL;
#endif

#ifdef WIN32
  char                  Buf[80];
#endif

  if ((Iniz = (float *)Alloca(Info -> Frames * sizeof(float)))) {

    /**** Initialization ****/

    switch (Par -> Com) {
    case MS_PSA:
    case MS_SURFACE:
    case MS_SURFDIA:
      if ((VdwRad = AssignVdwRad(InizAtm, MolAtm, Prefs.SAS_PROBERAD, &MaxRad))) {
        if ((SfUn = CalcSfUn(Prefs.SAS_POINTS, MaxRad, &Nu))) {
          if (!GLOBSW_STDOUT) {
            EneMin = CalcSrfArea(InizAtm, VdwRad, SfUn, Nu, Par, MaxRad, NULL);
            switch (Par -> Com) {
            case MS_SURFACE:
              k = MSG_DCD_STARTAREA;
              break;
            case MS_SURFDIA:
              k      = MSG_DCD_STARTSURFDIA;
              EneMin = SQR(EneMin / PI);
              break;
            case MS_PSA:
              k = MSG_DCD_STARTPSA;
              break;
            } /* End of switch */
            CatPrintf(stdout, k, EneMin);
          }
        } else Ret = FALSE;
      } else Ret = FALSE;
      break;

    case MS_VLOGP:
#ifdef VLOGP
      if ((VdwRad = AssignVdwRad(InizAtm, MolAtm, VLOGP_PROBE_RAD, &MaxRad))) {
        if ((SfUn = CalcSfUn(VLOGP_DOT_DENSITY, MaxRad, &Nu))) {
          if ((Contrib = (float *)Alloca(MolAtm * sizeof(float)))) {
            if (!GLOBSW_OPENGL) FindRing(InizAtm);
            PrintProg(MSG_DCD_ASSATMTYPES);
            if ((Ret = LogPCalc(InizAtm, MolAtm, &EneMax, &EneMin, "BROTO", Contrib))) {
              if (!GLOBSW_STDOUT)
                CatPrintf(stdout, MSG_DCD_STARTVLOGP,
                          CalcVLogP(InizAtm, VdwRad, MaxRad, SfUn, Nu, Contrib));
            } else Ret = FALSE;
          } else Ret = FALSE;
        } else Ret = FALSE;
      } else Ret = FALSE;
#else
      Ret = PrintVlogPErr();
#endif
      break;

    case MS_VOLUME:
    case MS_VOLDIA:
        if ((VdwRad = AssignVdwRad(InizAtm, MolAtm, 0.0, &MaxRad))) {
          CalcBox(InizAtm, VdwRad, &BoxMin, &BoxMax);
          if (!GLOBSW_STDOUT) {
            EneMin = CalcVolume(InizAtm, VdwRad, Prefs.VOL_DEN, &BoxMin, &BoxMax);
            if (Par -> Com == MS_VOLUME)
              CatPrintf(stdout, MSG_DCD_STARTVOL, EneMin);
            else
              CatPrintf(stdout, MSG_DCD_STARTVOLDIA, pow(6.0 * EneMin / PI, 1.0 / 3.0));
          }
        } else Ret = FALSE;
      break;

    case MS_DIPOLE:
      if (!ChkCharges(InizAtm, MolAtm, NULL))
        Ret = CatErr(MSG_ERR_DCD_UNCHARGED);
      break;
    } /* End of switch */

    if (Ret) {

      /**** Select atoms ****/

      for(k = 0; k < (VG_ULONG)Par -> Num; ++k) {
        for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
          if (Atm -> Num == Par -> Atm[k].Num) {
            Frm[k] = Atm;
            break;
          }
        } /* End of atom loop */
      } /* End of selection loop */

      /**** Make the calculations ****/

      ProcFrmTxt = GetStr(MSG_DCD_PROCFRAME);

#ifdef WIN32
      if (GLOBSW_OPENGL)
        ProgBarInit(GetStr(MSG_PRGBAR_TITLE), Info -> Frames);
      else
        YPos = wherey();
#else
      YPos = 0;
#endif

      Ene = Iniz;
      for(k = 1;(Ret) &&
                (k <= Info -> Frames) &&
                (Ret = TrjReadFrm(Info, k)); ++k) {
        GL_DrawScene(&ViewPrefs, TRUE);
        switch(Par -> Com) {
        case MS_DISTANCE:
          *Ene = SQR(Quad(Frm[0] -> x - Frm[1] -> x) + Quad(Frm[0] -> y - Frm[1] -> y) +
                     Quad(Frm[0] -> z - Frm[1] -> z));
          break;

        case MS_ANGLE:
          *Ene = (float)NormAngle(BondAngle(Frm[0], Frm[1], Frm[2]));
          break;

        case MS_TORSION:
          *Ene = (float)NormAngle(Torsion(Frm[0], Frm[1], Frm[2], Frm[3]));
          break;

        case MS_PLANEANG:
          *Ene = (float)NormAngle(PlaneAngAtm((ATOMO *)Frm) * RAD_TO_DEG);
          break;

#ifdef VLOGP
        case MS_VLOGP:
          *Ene = (float)CalcVLogP(InizAtm, VdwRad, MaxRad, SfUn, Nu, Contrib);
          break;
#endif
        case MS_VOLUME:
        case MS_VOLDIA:
          CalcBox(InizAtm, VdwRad, &BoxMin, &BoxMax);
          *Ene = (float)CalcVolume(InizAtm, VdwRad, Prefs.VOL_DEN, &BoxMin, &BoxMax);
          if (Par -> Com == MS_VOLDIA)
            *Ene = pow(6.0 * *Ene / PI, 1.0 / 3.0);
          break;

        case MS_DIPOLE:
          *Ene = (float)CalcDipole(InizAtm);
          break;

        case MS_SURFACE:
        case MS_SURFDIA:
          *Ene = (float)CalcSrfArea(InizAtm, VdwRad, SfUn, Nu, Par, MaxRad, NULL);
          if (Par -> Com == MS_SURFDIA) *Ene = SQR(*Ene / PI);
          break;

        case MS_PSA:
          CalcSrfArea(InizAtm, VdwRad, SfUn, Nu, Par, MaxRad, Ene);
          break;
        } /* End of switch */

        /**** Statistical values ****/

        Media  += *Ene;
        Media2 += Quad(*Ene);
        if (k == 1) {
          EneMin = *Ene;
          EneMax = *Ene;
        } else {
          if (*Ene < EneMin) {
            EneMin = *Ene;
            FrmMin = k;
          }
          if (*Ene > EneMax) {
            EneMax = *Ene;
            FrmMax = k;
          }
        }


        /**** Progress informations ****/

        if (!GLOBSW_STDOUT) {
#ifdef WIN32
          if (GLOBSW_OPENGL) {
            sprintf(Buf, ProcFrmTxt, k, Info -> Frames, *Ene);
            Ret = ProgBarUpd(Buf, k);
          } else {
            gotoxy(1, YPos);
            clreol();
            cprintf(ProcFrmTxt, k, Info -> Frames, *Ene);
          }
#else
          printf(ProcFrmTxt, k, Info -> Frames, *Ene);
          printf("          \n\033M");
#endif
        }
        ++Ene;
      } /* End of frame loop */


      if (!GLOBSW_STDOUT) {
#ifdef WIN32
        gotoxy(1, YPos);
        clreol();
#else
        printf("%40s\n\033M", " ");
#endif

        if (Ret)
          CatPrintf(stdout, MSG_DCD_RESULTS,
	  	    	    EneMin, FrmMin, EneMax, FrmMax,
                            Media / Info -> Frames,
                            SQR((Media2 - (Quad(Media) / Info -> Frames)) / (Info -> Frames - 1)));
#ifdef WIN32
        if (GLOBSW_OPENGL) {
          ProgBarClose();
          LocPrintf(stdout, "\n");
        }
#endif
      }
    }
  }

#ifdef VLOGP
  if (Contrib) FREE(Contrib);
#endif

  if (SfUn)    FREE(SfUn);
  if (VdwRad)  FREE(VdwRad);

  if (!Ret) {
    FREE(Iniz);
    Iniz = NULL;
  }

  return Iniz;
}


/**** Open a trajectory file ****/

void TrjClose(TRJINFO *Info)
{
  if (Info -> FH) fclose(Info -> FH);
  memset(Info, 0, sizeof(TRJINFO));
}


/**** Open a trajectory file ****/

VG_BOOL TrjOpen(char *FileName, TRJINFO *Info)
{
  VG_BOOL       Ret;

  if (TotalAtm) {
    memset(Info, 0, sizeof(TRJINFO));
    Ret = DCDOpen(FileName, Info);
  } else Ret = CatErr(MSG_ERR_DCD_NOMOL);

  return Ret;
}


/**** Save the data ****/

VG_BOOL TrjPlotSave(char *FileName, VG_ULONG Format, float *Data, VG_ULONG Tot)
{
  char          *OutMode;
  FILE          *FH;
  float         Ene, x;
  VG_ULONG      k;

  VG_BOOL       Ret = TRUE;


  /**** Set the write mode ****/

  switch (Format) {
  case FORM_PLOT_CSV:
  case FORM_PLOT_QUANTA:
    OutMode = "w";
    break;

  case FORM_PLOT_BIN:
    OutMode = "wb";
    break;

  default :
      return CatErr(MSG_ERR_DCD_UNSUPPOUT);
  }

  if ((FH = fopen(FileName, OutMode))) {
    if (!GLOBSW_OPENGL) PrintProg(MSG_ECD_SAVPLOT);
      for(k = 1; k <= Tot; ++k) {
        switch(Format) {
        case FORM_PLOT_CSV:           /* Comma separted values */
          if (fprintf(FH, "%5d; %14.6f\n", k, *Data) < 0)
            Ret = PrintDosErr();
          break;

        case FORM_PLOT_QUANTA:         /* Quanta plot          */
          if (fprintf(FH, "%5d.000000,%14.6f\n", k, *Data) < 0)
            Ret = PrintDosErr();
          break;

        case FORM_PLOT_BIN:            /* Binary Plot          */
          x = (float)k;
          Ene = *Data;
#ifdef LITTLE_ENDIAN
          Swap(&x);
          Swap(&Ene);
#endif
          if ((fwrite(&x   , sizeof(float), 1, FH) != 1) ||
              (fwrite(&Ene , sizeof(float), 1, FH) != 1))
            Ret = PrintDosErr();
          break;
        } /* End of switch */
        ++Data;
      } /* End of for */
    fclose(FH);
  } else Ret = PrintDosErr();

  return Ret;
}


/**** Read a specified frame ****/

VG_BOOL TrjReadFrm(TRJINFO *Info, VG_ULONG Frame)
{
  VG_BOOL       Ret = FALSE;

  if ((Info -> FH) && (Ret = TrjSeekFrm(Info, Frame - 1, SEEK_SET))) {
    switch(Info -> Format) {
    case FTRJ_DCD:      /* CHARMm DCD */
      Ret = DCDReadFrm(Info -> FH, BegAtm);
      break;
    } /* End of switch */
    if (Ret) ++Info -> FrmCur;
  }

  return Ret;
}


/**** Seek a trajectory frame ****/

VG_BOOL TrjSeekFrm(TRJINFO *Info, VG_LONG Frames, VG_LONG Mode)
{
  VG_BOOL       Ret;

  if (TotalAtm != Info -> MolAtm)
    return CatErr(MSG_ERR_DCD_UNKTRAJ);

  switch(Info -> Format) {
  case FTRJ_DCD:       /* CHARMm DCD */
    Ret = DCDSeekFrm(Info, Frames, Mode);
    break;

  default:
    Ret = FALSE;
  } /* End of switch */

  if (Ret) {
    switch(Mode) {
    case SEEK_CUR:
      Info -> FrmCur += Frames;
      break;

    case SEEK_END:
      Info -> FrmCur = Info -> Frames - Frames;
      break;

    case SEEK_SET:
      Info -> FrmCur = Frames;
      break;
    } /* End of switch */
  }

  return Ret;
}
